@@ -1,11 +1,10 @@
-from PyQt4.Qt import QApplication, QMessageBox, QDialog, QVBoxLayout, QLabel, QThread, SIGNAL
+from PyQt4.Qt import QApplication, QMessageBox, QDialog, QInputDialog, QLineEdit, QVBoxLayout, QLabel, QThread, SIGNAL
 import PyQt4.QtCore as QtCore
 from binascii import unhexlify
 from binascii import hexlify
 from struct import pack,unpack
 from sys import stderr
 from time import sleep
-from base64 import b64encode, b64decode
 
 import electrum
 from electrum_gui.qt.password_dialog import make_password_dialog, run_password_dialog
@@ -16,8 +15,9 @@ from electrum.plugins import BasePlugin, hook
 from electrum.transaction import deserialize
 from electrum.wallet import BIP32_HD_Wallet, BIP32_Wallet
 
-from electrum.util import format_satoshis_plain
+from electrum.util import format_satoshis_plain, print_error, print_msg
 import hashlib
+import threading
 
 try:
     from btchip.btchipComm import getDongle, DongleWait
@@ -28,7 +28,7 @@ try:
     from btchip.btchipFirmwareWizard import checkFirmware, updateFirmware
     from btchip.btchipException import BTChipException
     BTCHIP = True
-    BTCHIP_DEBUG = False 
+    BTCHIP_DEBUG = False
 except ImportError:
     BTCHIP = False
 
@@ -38,6 +38,7 @@ class Plugin(BasePlugin):
         BasePlugin.__init__(self, gui, name)
         self._is_available = self._init()
         self.wallet = None
+        self.handler = None
 
     def constructor(self, s):
         return BTChipWallet(s)
@@ -45,7 +46,7 @@ class Plugin(BasePlugin):
     def _init(self):
         return BTCHIP
 
-    def is_available(self):        
+    def is_available(self):
         if not self._is_available:
             return False
         if not self.wallet:
@@ -71,10 +72,20 @@ class Plugin(BasePlugin):
             return False
         return True
 
+    @hook
+    def cmdline_load_wallet(self, wallet):
+        self.wallet = wallet
+        self.wallet.plugin = self
+        if self.handler is None:
+            self.handler = BTChipCmdLineHandler()
+
     @hook
     def load_wallet(self, wallet, window):
         self.wallet = wallet
+        self.wallet.plugin = self
         self.window = window
+        if self.handler is None:
+            self.handler = BTChipQTHandler(self.window.app)
         if self.btchip_is_connected():
             if not self.wallet.check_proper_device():
                 QMessageBox.information(self.window, _('Error'), _("This wallet does not match your BTChip device"), _('OK'))
@@ -82,7 +93,7 @@ class Plugin(BasePlugin):
         else:
             QMessageBox.information(self.window, _('Error'), _("BTChip device not detected.\nContinuing in watching-only mode."), _('OK'))
             self.wallet.force_watching_only = True
-    
+
     @hook
     def installwizard_restore(self, wizard, storage):
         if storage.get('wallet_type') != 'btchip':
@@ -96,7 +107,7 @@ class Plugin(BasePlugin):
         return wallet
 
     @hook
-    def send_tx(self, tx):
+    def sign_tx(self, tx):
         tx.error = None
         try:
             self.wallet.sign_transaction(tx, None)
@@ -117,6 +128,7 @@ class BTChipWallet(BIP32_HD_Wallet):
         self.force_watching_only = False
 
     def give_error(self, message, clear_client = False):
+        print_error(message)
         if not self.signing:
             QMessageBox.warning(QDialog(), _('Warning'), _(message), _('OK'))
         else:
@@ -124,12 +136,16 @@ class BTChipWallet(BIP32_HD_Wallet):
         if clear_client and self.client is not None:
             self.client.bad = True
             self.device_checked = False
-        raise Exception(message)                
+        raise Exception(message)
 
     def get_action(self):
         if not self.accounts:
             return 'create_accounts'
 
+    def can_sign_xpubkey(self, x_pubkey):
+        xpub, sequence = BIP32_Account.parse_xpubkey(x_pubkey)
+        return xpub in self.master_public_keys.values()
+
     def can_create_accounts(self):
         return False
 
@@ -150,12 +166,12 @@ class BTChipWallet(BIP32_HD_Wallet):
 
         aborted = False
         if not self.client or self.client.bad:
-            try:
+            try:   
                 d = getDongle(BTCHIP_DEBUG)
-                d.setWaitImpl(DongleWaitQT(d))
                 self.client = btchip(d)
+                self.client.handler = self.plugin.handler                
                 firmware = self.client.getFirmwareVersion()['version'].split(".")
-                if not checkFirmware(firmware):                    
+                if not checkFirmware(firmware):
                     d.close()
                     try:
                         updateFirmware()
@@ -163,33 +179,31 @@ class BTChipWallet(BIP32_HD_Wallet):
                         aborted = True
                         raise e
                     d = getDongle(BTCHIP_DEBUG)
-                    d.setWaitImpl(DongleWaitQT(d))
-                    self.client = btchip(d)                    
+                    self.client = btchip(d)
                 try:
                     self.client.getOperationMode()
                 except BTChipException, e:
                     if (e.sw == 0x6985):
                         d.close()
-                        dialog = StartBTChipPersoDialog()                        
+                        dialog = StartBTChipPersoDialog()
                         dialog.exec_()
                         # Then fetch the reference again  as it was invalidated
                         d = getDongle(BTCHIP_DEBUG)
-                        d.setWaitImpl(DongleWaitQT(d))
                         self.client = btchip(d)
                     else:
                         raise e
-                if not noPin:                    
+                if not noPin:
                     # Immediately prompts for the PIN
-                    remaining_attempts = self.client.getVerifyPinRemainingAttempts()                    
+                    remaining_attempts = self.client.getVerifyPinRemainingAttempts()
                     if remaining_attempts <> 1:
                         msg = "Enter your BTChip PIN - remaining attempts : " + str(remaining_attempts)
                     else:
                         msg = "Enter your BTChip PIN - WARNING : LAST ATTEMPT. If the PIN is not correct, the dongle will be wiped."
-                    confirmed, p, pin = self.password_dialog(msg)                
+                    confirmed, p, pin = self.password_dialog(msg)
                     if not confirmed:
                         aborted = True
                         raise Exception('Aborted by user - please unplug the dongle and plug it again before retrying')
-                    pin = pin.encode()                   
+                    pin = pin.encode()
                     self.client.verifyPin(pin)
 
             except BTChipException, e:
@@ -197,18 +211,18 @@ class BTChipWallet(BIP32_HD_Wallet):
                     self.client.dongle.close()
                 except:
                     pass
-                self.client = None                
+                self.client = None
                 if (e.sw == 0x6faa):
-                    raise Exception("Dongle is temporarily locked - please unplug it and replug it again")                    
+                    raise Exception("Dongle is temporarily locked - please unplug it and replug it again")
                 if ((e.sw & 0xFFF0) == 0x63c0):
                     raise Exception("Invalid PIN - please unplug the dongle and plug it again before retrying")
                 raise e
             except Exception, e:
-                try:                 
+                try:
                     self.client.dongle.close()
                 except:
-                    pass                
-                self.client = None                                
+                    pass
+                self.client = None
                 if not aborted:
                     raise Exception("Could not connect to your BTChip dongle. Please verify access permissions, PIN, or unplug the dongle and plug it again")
                 else:
@@ -234,20 +248,20 @@ class BTChipWallet(BIP32_HD_Wallet):
         return []
 
     def get_public_key(self, bip32_path):
-        # S-L-O-W - we don't handle the fingerprint directly, so compute it manually from the previous node        
+        # S-L-O-W - we don't handle the fingerprint directly, so compute it manually from the previous node
         # This only happens once so it's bearable
-        self.get_client() # prompt for the PIN before displaying the dialog if necessary        
-        waitDialog.start("Computing master public key")
-        try:            
+        self.get_client() # prompt for the PIN before displaying the dialog if necessary
+        self.plugin.handler.show_message("Computing master public key")
+        try:
             splitPath = bip32_path.split('/')
-            fingerprint = 0        
+            fingerprint = 0
             if len(splitPath) > 1:
                 prevPath = "/".join(splitPath[0:len(splitPath) - 1])
                 nodeData = self.get_client().getWalletPublicKey(prevPath)
                 publicKey = compress_public_key(nodeData['publicKey'])
                 h = hashlib.new('ripemd160')
                 h.update(hashlib.sha256(publicKey).digest())
-                fingerprint = unpack(">I", h.digest()[0:4])[0]            
+                fingerprint = unpack(">I", h.digest()[0:4])[0]
             nodeData = self.get_client().getWalletPublicKey(bip32_path)
             publicKey = compress_public_key(nodeData['publicKey'])
             depth = len(splitPath)
@@ -255,12 +269,12 @@ class BTChipWallet(BIP32_HD_Wallet):
             if len(lastChild) == 1:
                 childnum = int(lastChild[0])
             else:
-                childnum = 0x80000000 | int(lastChild[0])        
+                childnum = 0x80000000 | int(lastChild[0])
             xpub = "0488B21E".decode('hex') + chr(depth) + self.i4b(fingerprint) + self.i4b(childnum) + str(nodeData['chainCode']) + str(publicKey)
         except Exception, e:
             self.give_error(e, True)
         finally:
-            waitDialog.emit(SIGNAL('dongle_done'))
+            self.plugin.handler.stop()
 
         return EncodeBase58Check(xpub)
 
@@ -270,7 +284,7 @@ class BTChipWallet(BIP32_HD_Wallet):
                 self.mpk = self.get_public_key("44'/0'")
             return self.mpk
         except Exception, e:
-            self.give_error(e, True)        
+            self.give_error(e, True)
 
     def i4b(self, x):
         return pack('>I', x)
@@ -287,13 +301,13 @@ class BTChipWallet(BIP32_HD_Wallet):
         self.signing = True
         self.get_client() # prompt for the PIN before displaying the dialog if necessary
         if not self.check_proper_device():
-            self.give_error('Wrong device or password')        
+            self.give_error('Wrong device or password')
         address_path = self.address_id(address)
-        waitDialog.start("Signing Message ...")
+        self.plugin.handler.show_message("Signing message ...")
         try:
             info = self.get_client().signMessagePrepare(address_path, message)
             pin = ""
-            if info['confirmationNeeded']:                
+            if info['confirmationNeeded']:
                 # TODO : handle different confirmation types. For the time being only supports keyboard 2FA
                 use2FA = True
                 confirmed, p, pin = self.password_dialog()
@@ -307,13 +321,12 @@ class BTChipWallet(BIP32_HD_Wallet):
         except BTChipException, e:
             if e.sw == 0x6a80:
                 self.give_error("Unfortunately, this message cannot be signed by BTChip. Only alphanumerical messages shorter than 140 characters are supported. Please remove any extra characters (tab, carriage return) and retry.")
-            else:                
-                self.give_error(e, True)            
+            else:
+                self.give_error(e, True)
         except Exception, e:
             self.give_error(e, True)
         finally:
-            if waitDialog.waiting:
-                waitDialog.emit(SIGNAL('dongle_done'))
+            self.plugin.handler.stop()
         self.client.bad = use2FA
         self.signing = False
 
@@ -331,23 +344,22 @@ class BTChipWallet(BIP32_HD_Wallet):
         s = str(s)
 
         # And convert it
-
-        return b64encode(chr(27 + 4 + (signature[0] & 0x01)) + r + s) 
+        return chr(27 + 4 + (signature[0] & 0x01)) + r + s
 
     def sign_transaction(self, tx, password):
         if tx.is_complete():
             return
-        if tx.error:
-            raise BaseException(tx.error)
-        self.signing = True        
+        #if tx.error:
+        #    raise BaseException(tx.error)
+        self.signing = True
         inputs = []
         inputsPaths = []
         pubKeys = []
         trustedInputs = []
-        redeemScripts = []        
+        redeemScripts = []
         signatures = []
         preparedTrustedInputs = []
-        changePath = "" 
+        changePath = ""
         changeAmount = None
         output = None
         outputAmount = None
@@ -358,8 +370,8 @@ class BTChipWallet(BIP32_HD_Wallet):
         for txinput in tx.inputs:
             if ('is_coinbase' in txinput and txinput['is_coinbase']):
                 self.give_error("Coinbase not supported")     # should never happen
-            inputs.append([ self.transactions[txinput['prevout_hash']].raw, 
-                             txinput['prevout_n'] ])        
+            inputs.append([ self.transactions[txinput['prevout_hash']].raw,
+                             txinput['prevout_n'] ])
             address = txinput['address']
             inputsPaths.append(self.address_id(address))
             pubKeys.append(self.get_public_keys(address))
@@ -367,7 +379,7 @@ class BTChipWallet(BIP32_HD_Wallet):
         # Recognize outputs - only one output and one change is authorized
         if len(tx.outputs) > 2: # should never happen
             self.give_error("Transaction with more than 2 outputs not supported")
-        for type, address, amount in tx.outputs:        
+        for type, address, amount in tx.outputs:
             assert type == 'address'
             if self.is_change(address):
                 changePath = self.address_id(address)
@@ -382,11 +394,11 @@ class BTChipWallet(BIP32_HD_Wallet):
         if not self.check_proper_device():
             self.give_error('Wrong device or password')
 
-        waitDialog.start("Signing Transaction ...")
+        self.plugin.handler.show_message("Signing Transaction ...")
         try:
             # Get trusted inputs from the original transactions
             for utxo in inputs:
-                txtmp = bitcoinTransaction(bytearray(utxo[0].decode('hex')))            
+                txtmp = bitcoinTransaction(bytearray(utxo[0].decode('hex')))
                 trustedInputs.append(self.get_client().getTrustedInput(txtmp, utxo[1]))
                 # TODO : Support P2SH later
                 redeemScripts.append(txtmp.outputs[utxo[1]].script)
@@ -394,46 +406,46 @@ class BTChipWallet(BIP32_HD_Wallet):
             firstTransaction = True
             inputIndex = 0
             while inputIndex < len(inputs):
-                self.get_client().startUntrustedTransaction(firstTransaction, inputIndex, 
+                self.get_client().startUntrustedTransaction(firstTransaction, inputIndex,
                 trustedInputs, redeemScripts[inputIndex])
-                outputData = self.get_client().finalizeInput(output, format_satoshis_plain(outputAmount), 
+                outputData = self.get_client().finalizeInput(output, format_satoshis_plain(outputAmount),
                 format_satoshis_plain(self.get_tx_fee(tx)), changePath, bytearray(rawTx.decode('hex')))
                 if firstTransaction:
                     transactionOutput = outputData['outputData']
-                if outputData['confirmationNeeded']:                
-                    use2FA = True
+                if outputData['confirmationNeeded']:                    
                     # TODO : handle different confirmation types. For the time being only supports keyboard 2FA
-                    waitDialog.emit(SIGNAL('dongle_done'))
+                    self.plugin.handler.stop()
                     if 'keycardData' in outputData:
                         pin2 = ""
                         for keycardIndex in range(len(outputData['keycardData'])):
                             msg = "Do not enter your device PIN here !\r\n\r\n" + \
                                 "Your BTChip wants to talk to you and tell you a unique second factor code.\r\n" + \
                                 "For this to work, please match the character between stars of the output address using your security card\r\n\r\n" + \
-                                "Output address : " 
+                                "Output address : "
                             for index in range(len(output)):
                                 if index == outputData['keycardData'][keycardIndex]:
                                     msg = msg + "*" + output[index] + "*"
                                 else:
                                     msg = msg + output[index]
-                            msg = msg + "\r\n"                        
+                            msg = msg + "\r\n"
                             confirmed, p, pin = self.password_dialog(msg)
                             if not confirmed:
-                                raise Exception('Aborted by user') 
+                                raise Exception('Aborted by user')
                             try:
                                 pin2 = pin2 + chr(int(pin[0], 16))
                             except:
                                 raise Exception('Invalid PIN character')
                         pin = pin2
                     else:
+                        use2FA = True
                         confirmed, p, pin = self.password_dialog()
                         if not confirmed:
-                            raise Exception('Aborted by user')                                            
+                            raise Exception('Aborted by user')
                         pin = pin.encode()
                         self.client.bad = True
                         self.device_checked = False
                         self.get_client(True)
-                    waitDialog.start("Signing ...")
+                    self.plugin.handler.show_message("Signing ...")
                 else:
                     # Sign input with the provided PIN
                     inputSignature = self.get_client().untrustedHashSign(inputsPaths[inputIndex],
@@ -445,14 +457,13 @@ class BTChipWallet(BIP32_HD_Wallet):
         except Exception, e:
             self.give_error(e, True)
         finally:
-            if waitDialog.waiting:
-                waitDialog.emit(SIGNAL('dongle_done'))
+            self.plugin.handler.stop()
 
         # Reformat transaction
         inputIndex = 0
         while inputIndex < len(inputs):
             # TODO : Support P2SH later
-            inputScript = get_regular_input_script(signatures[inputIndex], pubKeys[inputIndex][0].decode('hex'))        
+            inputScript = get_regular_input_script(signatures[inputIndex], pubKeys[inputIndex][0].decode('hex'))
             preparedTrustedInputs.append([ trustedInputs[inputIndex]['value'], inputScript ])
             inputIndex = inputIndex + 1
         updatedTransaction = format_transaction(transactionOutput, preparedTrustedInputs)
@@ -464,13 +475,13 @@ class BTChipWallet(BIP32_HD_Wallet):
     def check_proper_device(self):
         pubKey = DecodeBase58Check(self.master_public_keys["x/0'"])[45:]
         if not self.device_checked:
-            waitDialog.start("Checking device")
+            self.plugin.handler.show_message("Checking device")
             try:
                 nodeData = self.get_client().getWalletPublicKey("44'/0'/0'")
             except Exception, e:
                 self.give_error(e, True)
             finally:
-                waitDialog.emit(SIGNAL('dongle_done'))
+                self.plugin.handler.stop()
             pubKeyDevice = compress_public_key(nodeData['publicKey'])
             self.device_checked = True
             if pubKey != pubKeyDevice:
@@ -488,41 +499,69 @@ class BTChipWallet(BIP32_HD_Wallet):
                     "It should show itself to your computer as a keyboard and output the second factor along with a summary of the transaction it is signing into the text-editor.\r\n\r\n" \
                     "Check that summary and then enter the second factor code here.\r\n" \
                     "Before clicking OK, re-plug the device once more (unplug it and plug it again if you read the second factor code on the same computer)")
-        d = QDialog()
-        d.setModal(1)
-        d.setLayout( make_password_dialog(d, None, msg, False) )
-        return run_password_dialog(d, None, None)
+        response = self.plugin.handler.prompt_auth(msg)
+        if response is None:
+            return False, None, None
+        return True, response, response
 
-class DongleWaitingDialog(QThread):
-    def __init__(self):
-        QThread.__init__(self)
-        self.waiting = False
+class BTChipQTHandler:
 
-    def start(self, message):
+    def __init__(self, win):
+        self.win = win
+        self.win.connect(win, SIGNAL('btchip_done'), self.dialog_stop)
+        self.win.connect(win, SIGNAL('btchip_message_dialog'), self.message_dialog)
+        self.win.connect(win, SIGNAL('btchip_auth_dialog'), self.auth_dialog)
+        self.done = threading.Event()
+
+    def stop(self):
+        self.win.emit(SIGNAL('btchip_done'))
+
+    def show_message(self, msg):
+        self.message = msg
+        self.win.emit(SIGNAL('btchip_message_dialog'))
+
+    def prompt_auth(self, msg):
+        self.done.clear()
+        self.message = msg
+        self.win.emit(SIGNAL('btchip_auth_dialog'))
+        self.done.wait()
+        return self.response
+
+    def auth_dialog(self):
+        response = QInputDialog.getText(None, "BTChip Authentication", self.message, QLineEdit.Password)        
+        if not response[1]:
+            self.response = None
+        else:
+            self.response = str(response[0])
+        self.done.set()
+
+    def message_dialog(self):
         self.d = QDialog()
         self.d.setModal(1)
-        self.d.setWindowTitle('Please Wait')
+        self.d.setWindowTitle('BTChip')
         self.d.setWindowFlags(self.d.windowFlags() | QtCore.Qt.WindowStaysOnTopHint)
-        l = QLabel(message)
+        l = QLabel(self.message)
         vbox = QVBoxLayout(self.d)
         vbox.addWidget(l)
         self.d.show()
-        if not self.waiting:
-            self.waiting = True
-            self.d.connect(waitDialog, SIGNAL('dongle_done'), self.stop)
+
+    def dialog_stop(self):
+        if self.d is not None:
+            self.d.hide()
+            self.d = None
+
+class BTChipCmdLineHandler:
 
     def stop(self):
-        self.d.hide()
-        self.waiting = False
+        pass
 
-if BTCHIP:
-    waitDialog = DongleWaitingDialog()
-
-    # Tickle the UI a bit while waiting
-    class DongleWaitQT(DongleWait):
-        def __init__(self, dongle):
-            self.dongle = dongle
-
-        def waitFirstResponse(self, timeout):
-	    return self.dongle.waitFirstResponse(timeout)
+    def show_message(self, msg):
+        print_msg(msg)
 
+    def prompt_auth(self, msg):
+        import getpass        
+        print_msg(msg)
+        response = getpass.getpass('')
+        if len(response) == 0:
+            return None
+        return response
